@@ -23,7 +23,7 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-;;; (Mostly) Synched up with: FSF 20.3
+;;; (Mostly) Synched up with: FSF 20.5
 ;;; XEmacs has extra function `diary-countdown'
 
 ;;; Commentary:
@@ -294,7 +294,7 @@ These hooks have the following distinct roles:
                            (concat
                             "\\*\\|0*" (int-to-string year)
                             (if abbreviated-calendar-year
-                                (concat "\\|" (int-to-string (% year 100)))
+				(concat "\\|" (format "%02d" (% year 100)))
                               "")))
                           (regexp
                            (concat
@@ -395,6 +395,10 @@ changing the variable `diary-include-string'."
          (msg (format "No diary entries for %s %s"
                       (concat date-string (if holiday-list ":" ""))
                       (mapconcat 'identity holiday-list "; "))))
+    (calendar-set-mode-line
+     (concat "Diary for " date-string
+	     (if holiday-list ": " "")
+	     (mapconcat 'identity holiday-list "; ")))
     (if (or (not diary-entries-list)
             (and (not (cdr diary-entries-list))
                  (string-equal (car (cdr (car diary-entries-list))) "")))
@@ -410,10 +414,6 @@ changing the variable `diary-include-string'."
           (setq buffer-read-only t)
           (display-buffer holiday-buffer)
           (message  "No diary entries for %s" date-string))
-      (calendar-set-mode-line
-       (concat "Diary for " date-string
-               (if holiday-list ": " "")
-               (mapconcat 'identity holiday-list "; ")))
       (display-buffer (find-buffer-visiting d-file))
       (message "Preparing diary...done"))))
 
@@ -634,7 +634,7 @@ to run it every morning at 1am."
        (setq ndays diary-mail-days))
     (calendar)
     (view-diary-entries ndays)
-    (set-buffer "*Fancy Diary Entries*")
+    (set-buffer fancy-diary-buffer)
     (setq text (buffer-substring (point-min) (point-max)))
 
     ;; Now send text as a mail message.
@@ -769,7 +769,8 @@ After the entries are marked, the hooks `nongregorian-diary-marking-hook' and
                                      (string-to-int y-str)))))
                         (if dd-name
                             (mark-calendar-days-named
-                             (cdr (assoc (capitalize (substring dd-name 0 3))
+                             (cdr (assoc-ignore-case
+				   (substring dd-name 0 3)
                                          (calendar-make-alist
                                           calendar-day-name-array
                                           0
@@ -778,9 +779,8 @@ After the entries are marked, the hooks `nongregorian-diary-marking-hook' and
                               (if (string-equal mm-name "*")
                                   (setq mm 0)
                                 (setq mm
-                                      (cdr (assoc
-                                            (capitalize
-                                             (substring mm-name 0 3))
+                                      (cdr (assoc-ignore-case
+                                             (substring mm-name 0 3)
                                             (calendar-make-alist
                                              calendar-month-name-array
                                              1
@@ -844,10 +844,14 @@ is marked.  See the documentation for the function `list-sexp-diary-entries'."
               (backward-char 1)
               (setq entry ""))
           (setq entry-start (point))
+          ;; Find end of entry
           (re-search-forward "\^M\\|\n" nil t)
           (while (looking-at " \\|\^I")
-            (re-search-forward "\^M\\|\n" nil t))
-          (backward-char 1)
+	    (or (re-search-forward "\^M\\|\n" nil t)
+		(re-search-forward "$" nil t)))
+          (if (or (char-equal (preceding-char) ?\^M)
+		  (char-equal (preceding-char) ?\n))
+	      (backward-char 1))
           (setq entry (buffer-substring-no-properties entry-start (point)))
           (while (string-match "[\^M]" entry)
             (aset entry (match-beginning 0) ?\n )))
@@ -1318,9 +1322,10 @@ An optional parameter DAY means the Nth DAYNAME on or after/before MONTH DAY."
                                         (calendar-last-day-of-month m1 y1)))))
                        (and (<= d1 d) (<= d d2))))
                 ;; only possible base dates straddle two months
-                (and (< m1 m2)
+                (and (or (< y1 y2)
+                         (and (= y1 y2) (< m1 m2)))
                      (or
-                      ;; m1, d1 works is a base date
+                      ;; m1, d1 works as a base date
                       (and
                        (or (and (listp month) (memq m1 month))
                            (eq month t)
@@ -1328,7 +1333,7 @@ An optional parameter DAY means the Nth DAYNAME on or after/before MONTH DAY."
                        (<= d1 (or day (if (> n 0)
                                           1
                                         (calendar-last-day-of-month m1 y1)))))
-                      ;; m2, d2 works is a base date
+                      ;; m2, d2 works as a base date
                       (and (or (and (listp month) (memq m2 month))
                                (eq month t)
                                (= m2 month))
@@ -1415,27 +1420,32 @@ occur on.  If the current date is (one of) DAYS before the event indicated by
 SEXP, then a suitable message (as specified by `diary-remind-message' is
 returned.
 
-In addition to the reminders beforehand, the diary entry also appears on
-the date itself.
+In addition to the reminders beforehand, the diary entry also appears on the
+ date itself.
  
-If optional parameter MARKING is non-nil then the reminders are marked on the
-calendar.  Marking of reminders is independent of whether the entry itself is
-a marking or nonmarking one."
-  (let ((diary-entry))
-    (if (or (not marking-diary-entries) marking)
+A `diary-nonmarking-symbol' at the beginning of the line of the diary-remind
+entry specifies that the diary entry (not the reminder) is non-marking.
+Marking of reminders is independent of whether the entry itself is a marking
+or nonmarking; if optional parameter MARKING is non-nil then the reminders are
+marked on the calendar."
+  (let ((diary-entry (eval sexp)))
         (cond
-         ((integerp days)
+     ;; Diary entry applies on date
+     ((and diary-entry
+           (or (not marking-diary-entries) marking-diary-entry))
+      diary-entry)
+     ;; Diary entry may apply to `days' before date
+     ((and (integerp days)
+           (not diary-entry); Diary entry does not apply to date
+           (or (not marking-diary-entries) marking))
           (let ((date (calendar-gregorian-from-absolute
                        (+ (calendar-absolute-from-gregorian date) days))))
             (if (setq diary-entry (eval sexp))
-                (setq diary-entry (mapconcat 'eval diary-remind-message "")))))
+            (mapconcat 'eval diary-remind-message ""))))
+     ;; Diary entry may apply to one of a list of days before date
          ((and (listp days) days)
-          (setq diary-entry (diary-remind sexp (car days) marking))
-          (if (not diary-entry)
-              (setq diary-entry (diary-remind sexp (cdr days) marking))))))
-    (or diary-entry
-        (and (or (not marking-diary-entries) marking-diary-entry)
-             (eval sexp)))))
+      (or (diary-remind sexp (car days) marking)
+          (diary-remind sexp (cdr days) marking))))))
 
 (defun add-to-diary-list (date string specifier)
   "Add the entry (DATE STRING SPECIFIER) to `diary-entries-list'.
